/****************************************************************************
 * This file is part of the program mcmcalgn for joint sampling alignments  *
 * and mutation parameter values for two given DNA-sequences.               *
 *                                                                          *
 *    Copyright (C) 2000  Dirk Metzler                                      *
 *    e-mail: dmetzler@math.uni-frankfurt.de                                *  
 *                                                                          *
 *  This program is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation; either version 2 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program; if not, write to the                           *
 *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,             *
 *  Boston, MA  02111-1307  USA                                             *
 ****************************************************************************/

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

using namespace std;

#include "eingabe.hh"
#include "dnalign.hh"

void eingabe_parameter::interactive()
    /*
     * Interaktive Eingabe der eingabe_parameter
     */
{
    cout << "input file for sequence 1         : ";
    cin >> dnadat1;
    cout << "input file for sequence 2         : ";
    cin >> dnadat2;
    cout << "output file for parameter samples : ";
    cin >> param_samples;
    start_align=-1;
    while ( start_align<0 || start_align>2) {
	cout << "please choose number of initial alignment type :\n"
	     << " 0 => alignment given in input files\n"
	     << " 1 => sample alignment for parameters given below\n"
	     << " 2 => most probable alignment for parameters given below\n";
	cin >> start_align;
    }
    cout << "substitution rate                 : ";
    cin >> s;
    cout << "insertion rate                    : ";
    cin >> lambda;
    mu = lambda*1.001;
    cout << "no. of runs ...\n"
	 << "... during burn-in                : ";
    cin >> anz_burn;
    cout << "... after burn-in                 : ";
    cin >> anz_runs;
    cout << "... in each sampling interval     : ";
    cin >> anz_steps;
    cout << "size of resampling range          : ";
    cin >> resamp;
};

void eingabe_parameter::from_file(char * dateiname)
    /*
     * Eingabe der eingabe_parameter aus Datei dateiname 
     */
{
    ifstream inputstr(dateiname);
    if (!inputstr) {
	cerr << "cannot open file " << dateiname << endl;
	exit(EXIT_FAILURE);
    }
    inputstr >> dnadat1 >> dnadat2 >> param_samples 
	     >> start_align >> s >> lambda
	     >> anz_burn >> anz_runs >> anz_steps 
	     >> resamp ;
    inputstr.close();
    mu=lambda*1.001;
};
