/****************************************************************************
 * This file is part of the program mcmcalgn for joint sampling alignments  *
 * and mutation parameter values for two given DNA-sequences.               *
 *                                                                          *
 *    Copyright (C) 2000  Dirk Metzler                                      *
 *    e-mail: dmetzler@math.uni-frankfurt.de                                * 
 *                                                                          *
 *  This program is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation; either version 2 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program; if not, write to the                           *
 *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,             *
 *  Boston, MA  02111-1307  USA                                             *
 ****************************************************************************/

#include <vector>
#include <iostream>
#include <cstdio>
#include <fstream>
#include <cstdlib>
#include <ctime>
#include <string>

using namespace std;

#include "eingabe.hh"
#include "dnalign.hh"

int main(int argc,char* argv[]) {

    eingabe_parameter ep;
    
    if (argc==1) 
	ep.interactive(); 
    else {
	ep.from_file(argv[1]);
	if (argc<=2) 
	    srand(time(NULL));
	else {
	    unsigned int seed;
	    sscanf(argv[2],"%u",&seed);
	    srand(seed);
	}
    }
    dna_with_gaps dnawg_1(ep,1), dnawg_2(ep,2);
    dna dna_1(dnawg_1), dna_2(dnawg_2);
    alignment align(dna_1,dna_2,dnawg_1,dnawg_2,ep);
    cout << "initial alignment:\n"
	 << align;
    ofstream paramstr(ep.param_samples.c_str());
    align.print_params(paramstr);  
    align.print_path(paramstr); 
    paramstr << endl;
    alignment max_wk_align(align);
    cout << "the burn-in phase follows. please be patient ... \n";
    for( int i=0; i<ep.anz_burn; i++) {
	// burn in
	align.param_resamp();
	if (align.get_ln_wk() > max_wk_align.get_ln_wk())
	    max_wk_align=align;
	align.teilsample(ep.resamp);
	if (align.get_ln_wk() > max_wk_align.get_ln_wk()) 
	    max_wk_align=align ;
    }
    align.print_params(paramstr);  
    align.print_path(paramstr); 
    paramstr << endl;
    for( int i=0; i<ep.anz_runs; i++) {
	align.param_resamp();
	if (align.get_ln_wk() > max_wk_align.get_ln_wk() )
	    max_wk_align=align;
	align.teilsample(ep.resamp);
	if (align.get_ln_wk() > max_wk_align.get_ln_wk() )
	    max_wk_align=align;
	if ( i%ep.anz_steps == ep.anz_steps-1 ) {
	    align.print_params(paramstr);  
	    align.print_path(paramstr); 
	    paramstr << endl;
	    cout << endl << "alignment no. "<< i 
		 << endl << align
		;
	}
    }
    cout << "The most probable of all sampled alignments was: " << endl 
	 << max_wk_align;
    max_wk_align.print_params(paramstr);  
    max_wk_align.print_path(paramstr); 
    paramstr << endl;
    paramstr.close();
    return 0;
}

