#!/usr/bin/env wish
#
#  This file is part of the program mcmcalgn for joint sampling alignments  
#  and mutation parameter values for two given DNA-sequences.               
#                                                                         
#     Copyright (C) 2000  Dirk Metzler                                      
#     e-mail: dmetzler@math.uni-frankfurt.de                                 
#                                                                         
#  This program is free software; you can redistribute it and/or modify    
#  it under the terms of the GNU General Public License as published by    
#  the Free Software Foundation; either version 2 of the License, or       
#  (at your option) any later version.                                     
#                                                                        
#  This program is distributed in the hope that it will be useful,         
#  but WITHOUT ANY WARRANTY; without even the implied warranty of          
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           
#  GNU General Public License for more details.                            
#                                                                          
#  You should have received a copy of the GNU General Public License       
#  along with this program; if not, write to the                           
#  Free Software Foundation, Inc., 59 Temple Place, Suite 330,             
#  Boston, MA  02111-1307  USA                                             
#

wm minsize . 500 300

proc speichere {} {
    global seq1dat seq1dat
    global seq2dat seq2dat
    global ctrldat ctrldat
    global paramdat paramdat
    global start_algn start_algn
    global sub_rate sub_rate
    global ins_rate ins_rate
    global anz_burn anz_burn
    global anz_runs anz_runs
    global anz_steps anz_steps
    global resamp resamp 
    
    set ausgabe [open $ctrldat w]
    puts $ausgabe $seq1dat
    puts $ausgabe $seq2dat
    puts $ausgabe $paramdat
    puts $ausgabe [ switch $start_algn {
	given { format 0 }
	sampled { format 1 }
	ml { format 2 }
	default { puts "error in ausgabe!"} 
      }
    ]
    puts $ausgabe $sub_rate
    puts $ausgabe $ins_rate
    puts $ausgabe $anz_burn
    puts $ausgabe $anz_runs
    puts $ausgabe $anz_steps
    puts $ausgabe $resamp 
}

# Filenames

set seq1dat seq1
set seq2dat seq2
set ctrldat mcmcalgn.ctr
set paramdat paramsamp

frame .fr_files -relief sunken -borderwidth 1 
pack .fr_files  -fill x -expand 1
frame .fr_files.1st
frame .fr_files.2nd
pack .fr_files.1st .fr_files.2nd -in .fr_files 
frame .fr_files.3rd
frame .fr_files.4th
pack .fr_files.3rd .fr_files.4th -in .fr_files
label .l_seq1 -text "file with sequence 1" -width 30
entry .e_seq1 -textvariable seq1dat -width 30 -bg white
button .b_seq1 -text browse -command \
    {set seq1dat [tk_getOpenFile]}
pack .b_seq1 .e_seq1  .l_seq1 -side right -in .fr_files.1st
label .l_seq2 -text "file with sequence 2" -width 30
entry .e_seq2 -textvariable seq2dat -width 30 -bg white
button .b_seq2 -text browse -command \
    {set seq2dat [tk_getOpenFile]}
pack  .b_seq2 .e_seq2 .l_seq2 -side right -in .fr_files.2nd
label .l_ctrl -text "file for control instructions" -width 30
entry .e_ctrl -textvariable ctrldat -width 30 -bg white
button .b_ctrl -text browse -command \
    {set ctrldat [tk_getSaveFile -initialfile $ctrldat]}
pack .b_ctrl .e_ctrl  .l_ctrl -in .fr_files.3rd -side right
label .l_out -text "output file for parameters samples" -width 30
entry .e_out -textvariable paramdat -width 30 -bg white
button .b_out -text browse -command \
    {set paramdat [tk_getSaveFile -initialfile $paramdat]}
pack .b_out .e_out .l_out -in .fr_files.4th -side right

# Start Alignment

frame .fr_start -relief sunken -borderwidth 1
pack .fr_start  -fill x
frame .fr_start.align 
pack .fr_start.align -in  .fr_start -side left -fill x -expand 1
label .l_start_align -text "Start with..." -anchor w
pack .l_start_align -in .fr_start.align -fill x
set start_algn given
radiobutton .rb_given -variable start_algn -value given \
    -text "... given alignment" -anchor w 
radiobutton .rb_sampled -variable start_algn -value sampled \
    -text "... sampled alignment" -anchor w
radiobutton .rb_ml -variable start_algn -value ml \
    -text "... most probable alignment" -anchor w 
pack .rb_given .rb_sampled .rb_ml -in .fr_start.align  -fill x

# Start Parameters
frame .fr_start.param 
pack .fr_start.param -in  .fr_start -side left -expand 1 -fill x  -fill y 
label .l_start_param -text "Start parameters"
pack .l_start_param -in .fr_start.param -fill x 
frame .fr_start.param.1 
frame .fr_start.param.2
pack .fr_start.param.1 .fr_start.param.2 -in .fr_start.param -fill y -anchor e
label .l_sub -text "substitution rate"
set sub_rate 0.04
entry .e_sub -textvariable sub_rate -bg white -width 10
pack .e_sub  -in .fr_start.param.1 -side right -anchor e
pack .l_sub  -in .fr_start.param.1 -side right  -expand 1 -fill y 
label .l_indel -text "insertion rate"
set ins_rate 0.001
entry .e_indel -textvariable ins_rate -bg white -width 10
pack  .e_indel -in .fr_start.param.2 -side right -anchor e
pack  .l_indel -in .fr_start.param.2 -side right  -expand 1 -fill y 

# Numbers of Runs

frame .fr_runs
pack .fr_runs -expand 1 -fill x
frame .fr_runs.anz -borderwidth 1 -relief sunken 
pack .fr_runs.anz -in .fr_runs -side left -fill x 
frame .fr_runs.1
frame .fr_runs.2
frame .fr_runs.3
frame .fr_runs.4
pack .fr_runs.1 .fr_runs.2 .fr_runs.3 .fr_runs.4 -in  .fr_runs.anz -fill x 
label .l_burn -text "number of burn-in runs : "
set anz_burn 10000
entry .e_burn -textvariable anz_burn -bg white
pack .e_burn .l_burn -in .fr_runs.1 -side right -fill x  -anchor e
label .l_runs -text "number of runs after burn-in : "
set anz_runs 100000
entry .e_runs -textvariable anz_runs -bg white
pack .e_runs .l_runs -in .fr_runs.2 -side right -fill x -anchor e
label .l_steps -text "number of runs for each output : "
set anz_steps 1000
entry .e_steps -textvariable anz_steps -bg white
pack .e_steps .l_steps -in .fr_runs.3 -side right -fill x -anchor e
label .l_resamp -text "range of resampling : "
set resamp 30
entry .e_resamp -textvariable resamp -bg white
pack .e_resamp .l_resamp -in .fr_runs.4 -side right -fill x -anchor e

# Save and Exit Buttons
frame .fr_runs.buttons -borderwidth 1 -relief sunken
pack .fr_runs.buttons -in .fr_runs -side left -expand 1  -fill y -fill x
button .savebutton -command speichere -text "save"
pack .savebutton -in .fr_runs.buttons -expand 1  -fill y -fill x
button .exitbutton -command {speichere; exit}  -text "save and exit"
pack .exitbutton -in .fr_runs.buttons -expand 1 -fill y -fill x
button .se_button -command exit -text "exit without saving"
pack .se_button -in .fr_runs.buttons -expand 1  -fill y -fill x




