#!/usr/bin/env wish
#
#  This file is part of the program mcmcalgn for joint sampling alignments  
#  and mutation parameter values for two given DNA-sequences.               
#                                                                         
#     Copyright (C) 2000  Dirk Metzler                                      
#     e-mail: dmetzler@math.uni-frankfurt.de                                 
#                                                                         
#  This program is free software; you can redistribute it and/or modify    
#  it under the terms of the GNU General Public License as published by    
#  the Free Software Foundation; either version 2 of the License, or       
#  (at your option) any later version.                                     
#                                                                        
#  This program is distributed in the hope that it will be useful,         
#  but WITHOUT ANY WARRANTY; without even the implied warranty of          
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           
#  GNU General Public License for more details.                            
#                                                                          
#  You should have received a copy of the GNU General Public License       
#  along with this program; if not, write to the                           
#  Free Software Foundation, Inc., 59 Temple Place, Suite 330,             
#  Boston, MA  02111-1307  USA                                             
#

proc draw_algn {algn color} {
    set x 0
    set y 0
    set liste {0 0}
    set width 2
    if {$color == "black"} {
	set width 1
    }
    for {set i 0} {$i<[string length $algn]} {incr i} {
	switch [string index $algn $i] {
	    d {
		set x [expr $x+10]  
		set y [expr $y+10]
	    }
	    v { set y [expr $y+10] }
	    h { set x [expr $x+10] }
	}
	lappend liste "$x $y"
    }
    return [eval ".sc.c create line [join $liste { }]\
                   -fill $color -width $width"]
}

proc write_algn {algn} {
    global sequenz1 sequenz1 
    global sequenz2 sequenz2
    global algntxt
    set x 0
    set y 0
    set gapseq1 {}
    set gapseq2 {}
    set algnstring {}
    for {set i 0} {$i<[string length $algn]} {incr i} {
	switch [string index $algn $i] {
	    d {
		append gapseq1 [string index $sequenz1 $x]
		append gapseq2 [string index $sequenz2 $y]
		incr x  
		incr y
	    }
	    h { 
		append gapseq2 [string index $sequenz2 $y]
		append gapseq1 _
		incr y 
	    }
	    v { 
		append gapseq1 [string index $sequenz1 $x]
		append gapseq2 _
		incr x 
	    }
	}
    }
    set n 0
    while {[expr $n*100]<=[string length $gapseq1]} {
	append algnstring [string range $gapseq1 [expr $n*100] [expr $n*100+99]]\n
	append algnstring [string range $gapseq2 [expr $n*100] [expr $n*100+99]]\n
	append algnstring \n
	incr n
    }
    set algntxt $algnstring 
}

proc balken {algn} {
    global sequenz1 sequenz1 
    global sequenz2 sequenz2
    global gesamtzahl gesamtzahl
    global d_zahl d_zahl
    global h_zahl h_zahl
    global v_zahl v_zahl

    set x 0
    set y 0
    set gapseq1 {}
    set gapseq2 {}
    for {set i 0} {[expr $i*100]<[string length $algn]} {incr i} {
	set maxi($i) 0
    }
    for {set i 0} {$i<[string length $algn]} {incr i} {
	switch [string index $algn $i] {
	    d {
		append gapseq1 [string index $sequenz1 $x]
		append gapseq2 [string index $sequenz2 $y]
		set balken($i) [expr ($gesamtzahl-$d_zahl([expr 10*$y],[expr 10*$x]))\
				    *100.0/$gesamtzahl]
		if {$maxi([expr $i/100])<$balken($i)} {
		    set maxi([expr $i/100]) $balken($i)
		} 
		incr x  
		incr y
	    }
	    h { 
		append gapseq2 [string index $sequenz2 $y]
		append gapseq1 _
		set balken($i) [expr ($gesamtzahl-$h_zahl([expr 10*$y],[expr 10*$x]))\
				    *100.0/$gesamtzahl]
		if {$maxi([expr $i/100])<$balken($i)} {
		    set maxi([expr $i/100]) $balken($i)
		}
		incr y 
	    }
	    v { 
		append gapseq1 [string index $sequenz1 $x]
		append gapseq2 _
		set balken($i) [expr ($gesamtzahl-$v_zahl([expr 10*$y],[expr 10*$x]))\
				    *100.0/$gesamtzahl]
		if {$maxi([expr $i/100])<$balken($i)} {
		    set maxi([expr $i/100]) $balken($i)
		}
		incr x 
	    }
	}
    }
    .tlc.c delete all
    set grundlinie 10
    for {set i 0} {[expr $i*100]<=[string length $gapseq1]} {incr i} {
	set grundlinie [expr $grundlinie+$maxi($i)+40]
	for {set n 0} {$n < $maxi($i)} {set n [expr $n+20]} {
	    .tlc.c create line 10 [expr $grundlinie-$n] 620 [expr $grundlinie-$n] 
	    .tlc.c create text 630 [expr $grundlinie-$n] -text $n\% -font {Fixed 10 normal}
	} 
	for {set n [expr $i*100]} {$n < [expr ($i+1)*100] && $n<[string length $algn]} {incr n} {
	    .tlc.c create rectangle [expr 10+6*($n%100)] $grundlinie \
		[expr 16+6*($n%100)] [expr $grundlinie-$balken($n)] -fill grey
	    .tlc.c create text [expr 13+6*($n%100)] [expr $grundlinie+11] \
		-text [string index $gapseq1 $n] -font {Fixed 10 normal}
	    .tlc.c create text [expr 13+6*($n%100)] [expr $grundlinie+22] \
		-text [string index $gapseq2 $n] -font {Fixed 10 normal}
	}
    }
    .tlc.c configure -height [expr $grundlinie+30]
}

proc registriere {algn} {
    global d_zahl d_zahl
    global v_zahl v_zahl
    global h_zahl h_zahl
    global max_zahl max_zahl
    set max_zahl 0
    set x 0
    set y 0
    for {set i 0} {$i<[string length $algn]} {incr i} {
	switch [string index $algn $i] {
	    d {
		if {[info exists d_zahl($x,$y)]} {
		    incr d_zahl($x,$y)
		    if {$d_zahl($x,$y)>$max_zahl} {
			set max_zahl $d_zahl($x,$y)
		}
		} else {
		    set d_zahl($x,$y) 1
		}
		set x [expr $x+10]  
		set y [expr $y+10]
	    }
	    v { 
		if {[info exists v_zahl($x,$y)]} {
		    incr v_zahl($x,$y)
		    if {$v_zahl($x,$y)>$max_zahl} {
			set max_zahl $v_zahl($x,$y)
		    }
		} else {
		    set v_zahl($x,$y) 1
		}
		set y [expr $y+10] 
	    }
	    h { 
		if {[info exists h_zahl($x,$y)]} {
		    incr h_zahl($x,$y)
		    if {$h_zahl($x,$y)>$max_zahl} {
			set max_zahl $h_zahl($x,$y)
		    }
		} else {
		    set h_zahl($x,$y) 1
		}
		set x [expr $x+10] 
	    }
	}
	lappend liste "$x $y"
    }
}

proc male_basic {} {
    global d_zahl d_zahl
    global v_zahl v_zahl
    global h_zahl h_zahl
    global max_zahl max_zahl
    foreach i [array names d_zahl] {
	scan $i "%d,%d" x y
	if {$d_zahl($i)>[expr 0.8*$max_zahl]} {
	    set option "-width 5"
	} elseif {$d_zahl($i)>[expr 0.6*$max_zahl]} {
	    set option "-width 4"
	} elseif {$d_zahl($i)>[expr 0.4*$max_zahl]} {
	    set option "-width 3"
	} elseif {$d_zahl($i)>[expr 0.2*$max_zahl]} {
	    set option "-width 2"
	} elseif {$d_zahl($i)>1} {
	    set option ""
	} else {
	    set option "-fill grey60"
	}
	eval ".sc.c create line $x $y [expr $x+10] [expr $y+10] \
	      $option -capstyle round"
    }
    foreach i [array names v_zahl] {
	scan $i "%d,%d" x y
	if {$v_zahl($i)>[expr 0.8*$max_zahl]} {
	    set option "-width 5"
	} elseif {$v_zahl($i)>[expr 0.6*$max_zahl]} {
	    set option "-width 4"
	} elseif {$v_zahl($i)>[expr 0.4*$max_zahl]} {
	    set option "-width 3"
	} elseif {$v_zahl($i)>[expr 0.2*$max_zahl]} {
	    set option "-width 2"
	} elseif {$v_zahl($i)>1} {
	    set option ""
	} else {
	    set option "-fill grey60"
	}
	eval ".sc.c create line $x $y $x [expr $y+10] \
              $option -capstyle round"
    }
    foreach i [array names h_zahl] {
	scan $i "%d,%d" x y
	if {$h_zahl($i)>[expr 0.8*$max_zahl]} {
	    set option "-width 5"
	} elseif {$h_zahl($i)>[expr 0.6*$max_zahl]} {
	    set option "-width 4"
	} elseif {$h_zahl($i)>[expr 0.4*$max_zahl]} {
	    set option "-width 3"
	} elseif {$h_zahl($i)>[expr 0.2*$max_zahl]} {
	    set option "-width 2"
	} elseif {$h_zahl($i)>1} {
	    set option ""
	} else {
	    set option "-fill grey60"
	}
	eval ".sc.c create line $x $y [expr $x+10] $y \
	      $option -capstyle round"
    }
}

if $argc<1 {
    puts "filename missing"
    exit
}

proc speichern {bild} {
    set datei [tk_getSaveFile]
    if {$datei!=""} {
	$bild postscript -file $datei
    }
}

proc dview x {
    .sc.c xview scroll $x units
    .sc.c yview scroll $x units
}

set ctrfile [open [lindex $argv 0] r]
gets $ctrfile datname
set dna1file [open $datname]
gets $dna1file sequenz1
gets $ctrfile datname
set dna2file [open $datname]
gets $dna2file sequenz2
gets $ctrfile datname
set paramfile [open $datname]
close $ctrfile
close $dna1file
close $dna2file
regsub -all \[^ACGTagct\] $sequenz1 "" sequenz1
regsub -all \[^ACGTagct\] $sequenz2 "" sequenz2
set n 0
set max_s 0.592964
set max_lambda 0.0832141
while {[gets $paramfile zeile($n)]>-1} {
    scan $zeile($n) "%f %f %f %s" s($n) lambda($n) log_wk alignment($n)
    if $s($n)>$max_s {set max_s $s($n)}
    if $lambda($n)>$max_lambda {set max_lambda $lambda($n)}
    incr n
}

set gesamtzahl $n

set lenseq1 0
set lenseq2 0

for {set i 0} {$i<[string length $alignment(0)]} {incr i} {
    switch [string index $alignment(0) $i] {
	d {
	    incr lenseq1
	    incr lenseq2
	}
	v { incr lenseq2 }
	h { incr lenseq1 }
    }
}

frame .fc
pack .fc
frame .sc
frame .fb
pack .fb -side bottom
pack .sc -side left -in .fc -expand yes -fill both -padx 1 -pady 1
canvas .sc.c -height 500 -width 500 -bg white -relief sunken \
    -scrollregion "-10 -10 [expr 10*$lenseq1+10] [expr 10*$lenseq2+10]" \
    -xscrollcommand ".sc.h set" -yscrollcommand ".sc.v set"
canvas .cparam -height 500 -width 500 -bg white -relief sunken 
button .bexit -text exit -command exit
scrollbar .sc.h -orient horiz -command ".sc.c xview"
scrollbar .sc.v -command ".sc.c yview"
pack .cparam -side top -in .fc
grid rowconfig .sc 0 -weight 1 -minsize 0
grid columnconfig .sc 0 -weight 1 -minsize 0
grid .sc.c -padx 1 -in .sc -pady 1 \
    -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
grid .sc.v -padx 1 -in .sc -pady 1 \
    -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
grid .sc.h -padx 1 -in .sc -pady 1 \
    -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
scrollbar .sd -command "dview" -orient horiz
button .bsave1 -text save -command "speichern .sc.c"
button .bsave2 -text save -command "speichern .cparam"

label .l1pos -width 10 -textvariable dna1pos
label .l2pos -width 10 -textvariable dna2pos
label .ldummy1 -width 14
label .ldummy2 -width 14

bind .sc.c  <Motion> {
    set dna1pos [expr (%x+int([lindex [.sc.h get] 0]*(20+$lenseq1*10)))/10]
    set dna2pos [expr (%y+int([lindex [.sc.v get] 0]*(20+$lenseq2*10)))/10]
}

bind .sc.c  <Leave> {
    set dna1pos ""
    set dna2pos ""
}

pack .l1pos .bsave1 .sd .l2pos -in .fb -side left
pack .bexit -in .fb -side left -padx 100
pack .ldummy1 .bsave2 .ldummy2 -in .fb -side left 

.cparam create line 20 485 20 5 -arrow last
.cparam create line 15 480 495 480 -arrow last
.cparam create line 15 20 20 20 30 10 
.cparam create line 480 475 480 485
.cparam create text 20 495 -text 0
.cparam create text  7 480 -text 0
.cparam create text 490 495 -text $max_s -anchor e
.cparam create text  30 10 -text $max_lambda -anchor w

set max_zahl 0
registriere $alignment(0)
for {set i 1} {$i<[expr $n-1]} {incr i} {
    set s($i) [expr round($s($i)/$max_s*460+20)]
    set lambda($i) [expr round(480-$lambda($i)/$max_lambda*460)]
    registriere $alignment($i)
    set punkt [.cparam create oval [expr $s($i)-3] \
		   [expr $lambda($i)-3] [expr $s($i)+3] [expr $lambda($i)+3] \
		  -outline black -fill white]
    .cparam bind $punkt <Button-1> "
        .cparam itemconfigure current -fill orange
        if \[info exists line($i) \] {
	    .sc.c delete \$line($i)
        } 
	set line($i) \[draw_algn $alignment($i) orange\] 
        write_algn $alignment($i)
        balken $alignment($i)
   "
    .cparam bind $punkt <Button-2> "
        if \[info exists line($i) \] {
	    .cparam itemconfigure current -fill white
	    .sc.c delete \$line($i)
        } 
        write_algn $alignment($i)
        balken $alignment($i)
    "
    .cparam bind $punkt <Button-3> "
	.cparam itemconfigure current -fill green
        if \[info exists line($i) \] {
	    .sc.c delete \$line($i)
        } 
	set line($i) \[draw_algn $alignment($i) green\] 
        write_algn $alignment($i)
        balken $alignment($i)
    "
    # if {[expr $i%10]==0} {update idletasks}
}
male_basic

toplevel .tl
label .tl.lalgntxt -textvariable algntxt -justify left -font {Fixed 10 normal}
write_algn $alignment(0)
pack .tl.lalgntxt
button .tl.b -text write -command {puts $algntxt}
pack .tl.b

set s([expr $n-1]) [expr round($s([expr $n-1])/$max_s*460+20)]
set lambda([expr $n-1]) [expr round(480-$lambda([expr $n-1])/$max_lambda*460)]
draw_algn $alignment([expr $n-1]) red
set punkt [.cparam create oval [expr $s([expr $n-1])-3] [expr $lambda([expr $n-1])-3] \
	       [expr $s([expr $n-1])+3] [expr $lambda([expr $n-1])+3] \
	       -outline black -fill red]

.cparam bind $punkt <Button-1> "
	.cparam itemconfigure current -fill orange
	draw_algn $alignment([expr $n-1]) orange
        write_algn $alignment([expr $n-1])
        balken $alignment([expr $n-1])
    "
.cparam bind $punkt <Button-2> "
	.cparam itemconfigure current -fill red
	draw_algn $alignment([expr $n-1]) red
        write_algn $alignment([expr $n-1])
        balken $alignment([expr $n-1])
    "
.cparam bind $punkt <Button-3> "
	.cparam itemconfigure current -fill green
	draw_algn $alignment([expr $n-1]) green
        write_algn $alignment([expr $n-1])
        balken $alignment([expr $n-1])
    "

set s(0) [expr round($s(0)/$max_s*460+20)]
set lambda(0) [expr round(480-$lambda(0)/$max_lambda*460)]
draw_algn $alignment(0) purple
set punkt [.cparam create oval [expr $s(0)-3] [expr $lambda(0)-3] \
	       [expr $s(0)+3] [expr $lambda(0)+3] -outline black -fill purple]

.cparam bind $punkt <Button-1> "
	.cparam itemconfigure current -fill orange
	draw_algn $alignment(0) orange
        write_algn $alignment(0)
        balken $alignment(0)
    "
.cparam bind $punkt <Button-2> "
	.cparam itemconfigure current -fill purple
	draw_algn $alignment(0) purple
        write_algn $alignment(0)
        balken $alignment(0)
    "
.cparam bind $punkt <Button-3> "
	.cparam itemconfigure current -fill green
	draw_algn $alignment(0) green
        write_algn $alignment(0)
        balken $alignment(0)
    "

toplevel .tlc
canvas .tlc.c -width 650 -bg white
button .tlc.b -text save -command "speichern .tlc.c"
balken  $alignment(0)
pack .tlc.c -side left
pack .tlc.b
